"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _reactNative = require("react-native");

var _styles = require("./styles");

var _reactNativeBackgroundTimer = _interopRequireDefault(require("react-native-background-timer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* eslint-disable no-bitwise */
const defaulProps = {
  style: {},
  textStyle: {},
  onTimes: _seconds => {},
  onPause: _seconds => {},
  onEnd: _seconds => {}
};
let interval = null;
let hours = 0;
let minute = 0;
let seconds = 0;
let currentSeconds = 0;

const TimerComponent = /*#__PURE__*/_react.default.forwardRef((props, ref) => {
  const {
    initialSeconds = 0,
    style,
    textStyle,
    fontFamily,
    autoStart = false,
    formatTime = 'hh:mm:ss',
    onEnd,
    onTimes,
    onPause
  } = props;
  const [key, setKey] = (0, _react.useState)(Math.random());
  (0, _react.useImperativeHandle)(ref, () => {
    return {
      start,
      pause,
      resume,
      stop
    };
  });
  const timer = (0, _react.useCallback)(() => {
    interval = _reactNativeBackgroundTimer.default.setInterval(() => {
      currentSeconds = currentSeconds + 1;

      if (seconds < 60) {
        seconds = seconds + 1;
      } else {
        seconds = 0;
        minute = minute + 1;
      }

      if (minute === 60) {
        minute = 0;
        hours = hours + 1;
      }

      if (onTimes) {
        onTimes(currentSeconds);
      }

      setKey(Math.random());
    }, 1000);
  }, [onTimes]);
  const initTime = (0, _react.useCallback)(iSeconds => {
    if (iSeconds >= 3600) {
      hours = ~~(iSeconds / 3600);
      const times = iSeconds % 3600;
      initTime(times);
    } else {
      if (iSeconds >= 60) {
        minute = ~~(iSeconds / 60);
        const times = iSeconds % 60;
        initTime(times);
      } else {
        seconds = iSeconds;
      }
    }
  }, []);
  const init = (0, _react.useCallback)(() => {
    currentSeconds = 0;
    hours = 0;
    minute = 0;
    seconds = 0;

    if (initialSeconds > 0) {
      initTime(initialSeconds);
    }

    clear();
    setKey(Math.random());
  }, [initTime, initialSeconds]);
  const start = (0, _react.useCallback)(() => {
    init();

    if (!interval) {
      timer();
    }
  }, [init, timer]);
  const pause = (0, _react.useCallback)(() => {
    clear();

    if (onPause) {
      onPause(currentSeconds);
    }
  }, [onPause]);

  const resume = () => {
    if (!interval) {
      timer();
    }
  };

  const stop = () => {
    if (onEnd) {
      onEnd(currentSeconds);
    }

    init();
    setKey(Math.random());
    clear();
  };

  const clear = () => {
    if (interval) {
      _reactNativeBackgroundTimer.default.clearInterval(interval);

      interval = null;
    }
  };

  (0, _react.useEffect)(() => {
    return () => {
      pause();
      init();
      setKey(Math.random());
    };
  }, [init, pause]);
  (0, _react.useEffect)(() => {
    if (initialSeconds > 0) {
      init();
    }

    setKey(Math.random());
  }, [init, initialSeconds]);
  (0, _react.useEffect)(() => {
    if (autoStart) {
      start();
    }
  }, [autoStart, initialSeconds, start]);

  const font = () => {
    if (fontFamily) {
      return {
        fontFamily: fontFamily
      };
    } else {
      return {};
    }
  };

  const renderTimer = () => {
    if (formatTime === 'hh:mm:ss') {
      if (hours > 0) {
        return /*#__PURE__*/_react.default.createElement(_reactNative.Text, {
          style: [_styles.styles.text, textStyle, font()]
        }, `${hours}:${minute.toString().length === 1 ? '0' : ''}${minute}:${seconds.toString().length === 1 ? '0' : ''}${seconds}`);
      } else {
        if (minute > 0) {
          return /*#__PURE__*/_react.default.createElement(_reactNative.Text, {
            style: [_styles.styles.text, textStyle, font()]
          }, `${minute}:${seconds.toString().length === 1 ? '0' : ''}${seconds}`);
        } else {
          return /*#__PURE__*/_react.default.createElement(_reactNative.Text, {
            style: [_styles.styles.text, textStyle, font()]
          }, `${seconds}`);
        }
      }
    } else {
      return /*#__PURE__*/_react.default.createElement(_reactNative.Text, {
        style: [_styles.styles.text, textStyle, font()]
      }, `${currentSeconds}`);
    }
  };

  return /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    style: style,
    key: key
  }, renderTimer());
});

TimerComponent.defaultProps = defaulProps;
var _default = TimerComponent;
exports.default = _default;
//# sourceMappingURL=index.js.map